// Connect speaker to pin 13 with 100ohm resistor
int speakerPin = 13;
int heartPin = 3;

// notes thanks to https://pages.mtu.edu/~suits/notefreqs.html
int Fnote = 349;
int Dnote = 587;
int Cnote = 523;
int Anote = 440;

// initialize BPM settings, beatTime, and lastTime
// these can be overriden in setup()
int targetBPM = 80;
int currentBPM = 160;
int matchBPM = 120;
unsigned long beatTime = 500;
int lastTime = 0;

// for LED feedback
#define LEDPIN RGB_LED 
#define NUMBER_PIEXELS 1 
#include <Adafruit_NeoPixel.h>
Adafruit_NeoPixel strip = Adafruit_NeoPixel(NUMBER_PIEXELS, LEDPIN, NEO_GRB + NEO_KHZ800);

void measureBPM(){
  if(lastTime != 0){
    // calculate current BPM based upon time since last beat
    currentBPM = 60000 / (millis() - lastTime);
  }

  // set time of this beat as last beat
  lastTime = millis();

  // set the matchBPM to halfway to target
  matchBPM = abs(currentBPM + targetBPM)/2;

  // calculate beat time from BPM
  beatTime = 60000/matchBPM;

  // send serial status of current BPMs
  SerialUSB.println();
  SerialUSB.println("Current   "+String(currentBPM));
  SerialUSB.println("Target    "+String(targetBPM));
  SerialUSB.println("Match     "+String(matchBPM));
  SerialUSB.println();
}

void setup() {
  // setup LED
  strip.begin();

  // setup heartbeat pin as interrupt
  pinMode(heartPin, INPUT_PULLUP);
  attachInterrupt(digitalPinToInterrupt(heartPin), measureBPM, FALLING);
  
  // initialize serial
  SerialUSB.begin(9600);

  // initialize/override target and current BPM
  // slow
//  targetBPM = 80;
//  currentBPM = 160;
  // fast
  targetBPM = 180;
  currentBPM = 100;

  // calculate initial matchBPM
  matchBPM = abs(currentBPM + targetBPM)/2;
  beatTime = 60000/matchBPM;

  SerialUSB.println("Current   "+String(currentBPM));
  SerialUSB.println("Target    "+String(targetBPM));
  SerialUSB.println("Match     "+String(matchBPM));
  SerialUSB.println();
}

void loop() {
  // play F6 arpeggio
  SerialUSB.println("F");
  tone(speakerPin, Fnote);
  strip.setPixelColor(0, strip.Color(255,255,255));
  strip.show();
  delay(beatTime);
  
  SerialUSB.println("D");
  tone(speakerPin, Dnote, beatTime);
  strip.setPixelColor(0, strip.Color(255,0,0));
  strip.show();
  delay(beatTime);
  
  
  SerialUSB.println("C");
  tone(speakerPin, Cnote, beatTime);
  strip.setPixelColor(0, strip.Color(0,255,0));
  strip.show();
  delay(beatTime);
  
  
  SerialUSB.println("A");
  tone(speakerPin, Anote, beatTime);
  strip.setPixelColor(0, strip.Color(0,0,255));
  strip.show();
  delay(beatTime);
  
}
